/*******************************************************************************

	HAMURABI - The Sumer Game

		C Port Version 1.00


		*The Sumer Game
			(C)1968 FOCAL  Version by Doug Dyment
		*HAMURABI
			(C)1978 BASIC  Version by David H.Ahl
			(C)2010 C Port Version by Piroyan

*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#if !defined(MSXDOS)
#include <time.h>
#endif

/*******************************************************************************
	DEFINE
*******************************************************************************/
typedef struct
{
	long		t_year ;
	long		year ;
	long		starved ;
	long		come ;
	long		people ;
	long		acre ;
	long		store ;
	long		eat ;

	long		d1 ;
	float		p1 ;
	float		t_app ;

	short		harvest ;
	short		rate ;

	long		rest ;
	long		acre_rest ;

	short		plague ;
	short		advice_mode ;

	/* input data */
	long		trade ;
	long		feed ;
	long		plant ;

	/* for ending */
	float		acre_per_people	;
	long		discontent ;

}rPARAM_t ;

typedef void (*INIT_FUNC)( rPARAM_t *prtPARAM ) ;
typedef int  (*EXEC_FUNC)( rPARAM_t *prtPARAM ) ;

/* message number */
enum{
	_MES_NULL_ = 0,

	_MES_NAME_,

	_MES_HOW_MANY_YEARS_,

	_MES_INTRO_1_,
	_MES_INTRO_2_,

	_MES_REPORT_,
	_MES_YEAR_,
	_MES_STARVED_,
	_MES_CAME_TO_,
	_MES_PLAGUE_,
	_MES_POPULATION_,
	/*   11   */
	_MES_NOW_OWNS_ACRES_,
	_MES_HARVESTED_,
	_MES_RATS_,
	_MES_STORE_,
	_MES_TRADING_,

	_MES_BUY_,
	_MES_SELL_,
	_MES_FEED_,
	_MES_PLANT_,

	_MES_SURE_,
	/*   21   */
	_MES_THINK_AGAIN_,
	_MES_NO_PEOPLE_,
	_MES_NO_BUSHELS_,
	_MES_NO_ACRES_,

	_MES_ABORT_1_,
	_MES_ABORT_2_,

	_MES_GAMEOVER_1_,
	_MES_GAMEOVER_2_,
	_MES_GAMEOVER_3_,
	_MES_GAMEOVER_4_,
	/*   31   */
	_MES_END_1_1_,
	_MES_END_1_2_,
	_MES_END_1_3_,
	_MES_END_1_4_,

	_MES_END_2_1_,
	_MES_END_2_2_,

	_MES_ENDING_1_1_,
	_MES_ENDING_1_2_,

	_MES_ENDING_2_1_,
	_MES_ENDING_2_2_,
	/*   41   */
	_MES_ENDING_2_3_,

	_MES_ENDING_3_1_,
	_MES_ENDING_3_2_,
	_MES_ENDING_3_3_,
	_MES_ENDING_3_4_,

	_MES_TRY_AGAIN_,

	_MES_SO_LONG_,

	_MES_ADVICE_,
	_MES_ADVICE_1_DEMAND_,
	_MES_ADVICE_2_PLANT_,
	_MES_ADVICE_3_TRADING_BUY_,
	_MES_ADVICE_3_TRADING_SELL_,
	_MES_ADVICE_4_RESULT_LAND_,
	_MES_ADVICE_4_RESULT_PLANT_,

	/* max number of message strings */
	MESSAGE_MAX
} ;

/* max input line (if undefined) */
#ifndef	MAX_LINE
#define	MAX_LINE	128
#endif

/* maximum number of variation */
#define	VARIATION_MAX		2
/* default number of variation */
#define	VARIATION_DEFAULT	0

/* maximum number of advice mode */
#define	ADVICE_MAX		0x03	/* 00000011b */
/* default number of advice mode */
#define	ADVICE_DEFAULT	0x00

/* Amount of bushel necessary for people (per person) */
#define	BUSHELS_PER_PERSON	20

/*******************************************************************************
	PROTOTYPE
*******************************************************************************/
void randomize_time( void ) ;
unsigned short get_rand( int num ) ;

int read_mesdata( char *fname ) ;
void usage( void ) ;

void game_main( rPARAM_t *prtPARAM, char variation ) ;

void title( void ) ;
void report( rPARAM_t *prtPARAM, char flag ) ;
void advice( rPARAM_t *prtPARAM ) ;

int input_num( rPARAM_t *prtPARAM, int mesno ) ;
char *input_str( int mesno ) ;
int set_param( rPARAM_t *prtPARAM ) ;

void ending( rPARAM_t *prtPARAM ) ;

/* variation : 000 */
void init_func_var_000( rPARAM_t *prtPARAM ) ;
int  exec_func_var_000( rPARAM_t *prtPARAM ) ;

/* variation : 001 */
void init_func_var_001( rPARAM_t *prtPARAM ) ;
int  exec_func_var_001( rPARAM_t *prtPARAM ) ;

void init( rPARAM_t *prtPARAM, char variation ) ;
int execute( rPARAM_t *prtPARAM, char variation ) ;

/*******************************************************************************
	RAM DATA
*******************************************************************************/
/* default message strings (English) */
static char MES_STRING[MESSAGE_MAX][MAX_LINE] =
{
	"",
	/* -------------------------------   01   ------------------------------- */
	"HAMURABI",
	"HOW MANY YEARS DO YOU GOVERN THE CITY? (1-100 / DEFAULT:%d)",
	"TRY YOUR HAND AT GOVERNING ANCIENT SUMERIA`",
	"FOR A %d-YEAR TERM OF OFFICE.`",
	"I BEG TO REPORT TO YOU,`",
	"IN YEAR %d, ",
	"%d PEOPLE STARVED, ",
	"%d CAME TO THE CITY.`",
	"A HORRIBLE PLAGUE STRUCK!  HALF THE PEOPLE DIED.`",
	"POPULATION IS NOW %d`",
	/* -------------------------------   11   ------------------------------- */
	"THE CITY NOW OWNS %d ACRES.`",
	"YOU HARVESTED %d BUSHELS PER ACRE.`",
	"RATS ATE %d BUSHELS.`",
	"YOU NOW HAVE %d BUSHELS IN STORE.`",
	"LAND IS TRADING AT %d BUSHELS PER ACRE.`",
	"HOW MANY ACRES DO YOU WISH TO BUY?",
	"HOW MANY ACRES DO YOU WISH TO SELL?",
	"HOW MANY BUSHELS DO YOU WISH TO FEED YOUR PEOPLE?",
	"HOW MANY ACRES DO YOU WISH TO PLANT WITH SEED?",
	"MAY I EXECUTE YOUR COMMANDS?",
	/* -------------------------------   21   ------------------------------- */
	"THINK AGAIN. ",
	"BUT YOU HAVE ONLY %d PEOPLE TO TEND THE FIELDS.  NOW THEN,`",
	"YOU HAVE ONLY`%d BUSHELS OF GRAIN.  NOW THEN,`",
	"YOU OWN ONLY %d ACRES.  NOW THEN,`",
	"I CANNOT DO WHAT YOU WISH.`",
	"GET YOURSELF ANOTHER STEWARD!!!!!`",
	"YOU STARVED %d PEOPLE IN ONE YEAR!!!`",
	"DUE TO THIS EXTREME MISMANAGEMENT YOU HAVE NOT ONLY`",
	"BEEN IMPEACHED AND THROWN OUT OF OFFICE BUT YOU HAVE`",
	"ALSO BEEN DECLARED 'NATIONAL FINK' !!`",
	/* -------------------------------   31   ------------------------------- */
	"IN YOUR %d-YEAR TERM OF OFFICE, ",
	"%2.2f PERCENT OF THE`",
	"POPULATION STARVED PER YEAR ON AVERAGE, I.E., A TOTAL OF` ",
	"%d PEOPLE DIED!!`",
	"YOU STARTED WITH %5.2f ACRES PER PERSON AND ENDED WITH` ",
	"%5.2f ACRES PER PERSON.`",
	"A FANTASTIC PERFORMANCE!!!  CHARLEMAGNE, DISRAELI, AND`",
	"JEFFERSON COMBINED COULD NOT HAVE DONE BETTER!`",
	"YOUR HEAVY-HANDED PERFORMANCE SMACKS OF NERO AND IVAN IV.`",
	"THE PEOPLE (REMAINING) FIND YOU AN UNPLEASANT RULER, AND,`",
	/* -------------------------------   41   ------------------------------- */
	"FRANKLY, HATE YOUR GUTS!`",
	"YOUR PERFORMANCE COULD HAVE BEEN SOMEWHAT BETTER, BUT`",
	"REALLY WASN'T TOO BAD AT ALL. %d PEOPLE WOULD`",
	"DEARLY LIKE TO SEE YOU ASSASSINATED BUT WE ALL HAVE OUR`",
	"TRIVIAL PROBLEMS.`",
	"TRY AGAIN?",
	"SO LONG FOR NOW.`",
	"ADVICE",
	"QUANTITIES OF BUSHEL NECESSARY FOR PEOPLE IS %d BUSHELS.`",
	"AREA THAT CAN BE PLANTED WITH SEED IS %d ACRES.`",
	/* -------------------------------   51   ------------------------------- */
	"AREA IN LAND THAT CAN BE BOUGHT IS %d ACRES.`",
	"FOR PEOPLE, AREA IN LAND THAT SHOULD BE SOLD IS %d ACRES.`",
	"...AS A RESULT, AREA IN LAND BECOMES %d ACRES.`",
	"AND, AREA THAT CAN BE PLANTED WITH SEED BECOMES %d ACRES.`",
} ;

static char		g_buf[256] ;

static unsigned long	RND_SEED ;

/* initialize function table */
static INIT_FUNC INITIALIZE[VARIATION_MAX] =
{
	init_func_var_000,
	init_func_var_001,
} ;

/* execute function table */
static EXEC_FUNC EXECUTE[VARIATION_MAX] =
{
	exec_func_var_000,
	exec_func_var_001,
} ;

/*******************************************************************************
	PROGRAM CODE
*******************************************************************************/

/*==============================================================================
	main routine
==============================================================================*/
int main( int argc, char** argv )
{
	int		t0 ;
	char	*p ;
	char	param ;
	char	variation ;

	rPARAM_t	rtPARAM ;

	variation = VARIATION_DEFAULT ;
	rtPARAM.advice_mode = 0 ;

	if ( argc > 1 )
	{
		printf( "\n" ) ;
		for ( param = 1 ; param < argc ; param++ )
		{
			p = argv[param] ;
			switch( *p )
			{
				case ' ':
				case '\0':
				case '\n':
					/* for MSX-DOS
						 ... argc count up even if argv is blank line */
					break ;

				case '-':
				case '/':
					switch ( *(++p))
					{
						/* Set Game Variation */
						case 'v':
						case 'V':
							variation = (char)atoi( ++p ) ;
							if ( variation < VARIATION_MAX )
							{
								printf(
									"*Set Game Variation : %d\n",
									variation ) ;
							}else{
								printf(
									"*ERROR : Max Variation is %d\n",
									VARIATION_MAX - 1 ) ;
								variation = VARIATION_DEFAULT ;
								printf( "  ... use default : %d\n",
									variation ) ;
							}
							break ;

						/* Set Advice Mode */
						case 'a':
						case 'A':
							rtPARAM.advice_mode = (char)atoi( ++p ) ;
							if ( rtPARAM.advice_mode <= ADVICE_MAX )
							{
								printf(
									"*Set Advice Mode : %d\n",
									rtPARAM.advice_mode ) ;
							}else{
								printf(
									"*ERROR : Max Advice Mode Number is %d\n",
									ADVICE_MAX ) ;
								rtPARAM.advice_mode = ADVICE_DEFAULT ;
								printf( "  ... use default : %d\n",
									rtPARAM.advice_mode ) ;
							}
							break ;

						/* Set Message Data */
						case 'm':
						case 'M':
							if ( read_mesdata( ++p ))
							{
								printf( "*Set Message Data : '%s'\n", p ) ;
							}else{
								printf( "*ERROR : '%s' is not found.\n", p ) ;
							}
							break ;

						default:
							usage() ;
							return 0 ;
					}
					break ;

				default:
					usage() ;
					return 0 ;
			}
		}
		printf( "\n" ) ;
	}

	for ( t0 = 0 ; t0 < MESSAGE_MAX ; t0++ )
	{
		p = MES_STRING[t0] ;
		do{
			if ( *p == '\n' )	*p = '\0' ;
			if ( *p == '`' )	*p = '\n' ;
		}
		while( *p++ != 0 ) ;
	}

	game_main( &rtPARAM, variation ) ;

	printf( "%s\n", MES_STRING[_MES_SO_LONG_] ) ;

	return 0 ;
}


/*==============================================================================
	sub routines
==============================================================================*/

/*------------------------------------------------------------------------------
	srand(time)
------------------------------------------------------------------------------*/
void randomize_time( void )
{
#if defined(MSXDOS)
/* Because time() function doesn't operate normally in MSX-DOS.. */
#asm
	ld		a, r
	ld		( _RND_SEED ), a
#endasm
#else
	RND_SEED = (unsigned long)time( NULL ) ;
#endif
}


/*------------------------------------------------------------------------------
	rand() % num
------------------------------------------------------------------------------*/
unsigned short get_rand( int num )
{
	RND_SEED = RND_SEED * 69069L + 1L ;
	return (unsigned short)( RND_SEED >> 16 ) % num ;
}


/*------------------------------------------------------------------------------
	read message data
------------------------------------------------------------------------------*/
int read_mesdata( char *fname )
{
	FILE	*fp ;
	int		i, j ;

	if (( fp = fopen( fname, "rt" )) != NULL )
	{
		while ( fgets( g_buf, MAX_LINE, fp ) != NULL )
		{
			if ( g_buf[0] == '#' || g_buf[0] =='\n' )
			{
				continue ;
			}
			for ( i = 0 ; ( g_buf[i] != ' ' && g_buf[i] != 0x09 ) ; i++ ) ;
			g_buf[i++] = '\0' ;
			j = atoi( g_buf ) ;
			if ( j > 0 && j < MESSAGE_MAX )
			{
				while ( g_buf[i] == ' ' || g_buf[i] == 0x09 ) i++ ;
				strcpy( MES_STRING[j], &g_buf[i] ) ;
			}
		}
	}else{
		return 0 ;
	}
	fclose( fp ) ;
	return 1 ;
}


/*------------------------------------------------------------------------------
	usage
------------------------------------------------------------------------------*/
void usage( void )
{
	title() ;
	printf( "usage :\n" ) ;
	printf(
		"hamurabi {-vVARIATION no.} {-aADVICE MODE no.} {-mMESSAGE DATA}\n\n" ) ;
	printf( "  VARIATION no.: 0-%d\n\n", VARIATION_MAX - 1 ) ;
	printf( "  ADVICE MODE no.: 0-%d\n", ADVICE_MAX ) ;
	printf( "    Enable display at...\n" ) ;
	printf( "     (bit) x1 ... Quantities of Bushels for People.\n" ) ;
	printf( "           1x ... Land that can be bought.\n" ) ;
}


/*------------------------------------------------------------------------------
	title & copyright
------------------------------------------------------------------------------*/
void title( void )
{
	printf( "\n               HAMURABI - The Sumer Game\n\n" ) ;
	printf( "The Sumer Game " ) ;
	printf( "(C)1968 FOCAL  Version by Doug Dyment\n" ) ;
	printf( "      HAMURABI " ) ;
	printf( "(C)1978 BASIC  Version by David H.Ahl\n" ) ;
	printf( "               " ) ;
	printf( "(C)2010 C Port Version by Piroyan\n\n\n" ) ;
}


/*------------------------------------------------------------------------------
	game main
------------------------------------------------------------------------------*/
void game_main( rPARAM_t *prtPARAM, char variation )
{
	long	t0 ;
	char	*p ;
	int		end_flag ;

	title() ;

	do{
		init( prtPARAM, variation ) ;

		prtPARAM->t_app = (float)prtPARAM->acre / prtPARAM->people ;
		do{
			printf( MES_STRING[_MES_HOW_MANY_YEARS_], prtPARAM->t_year ) ;
			fgets( g_buf, sizeof( g_buf ), stdin ) ;
			t0 = atoi( g_buf ) ;
		}
		while ( t0 < 0 || t0 > 100 ) ;
		if ( t0 > 0 )
		{
			prtPARAM->t_year = t0 ;
		}
		printf( "\n\n" ) ;

		/* introduction */
		printf( MES_STRING[_MES_INTRO_1_] ) ;
		printf( MES_STRING[_MES_INTRO_2_], prtPARAM->t_year ) ;

		/* game main */
		do{
			report( prtPARAM, 1 ) ;

			end_flag = set_param( prtPARAM ) ;

			if ( end_flag > 0 )
			{
				end_flag = execute( prtPARAM, variation ) ;
			}
		}
		while( end_flag > 0 ) ;

		/* ending */
		if ( end_flag == 0 )
		{
			report( prtPARAM, 1 ) ;
			ending( prtPARAM ) ;
		}

		/* again? */
		printf( "\n\n" ) ;
		p = input_str(_MES_TRY_AGAIN_) ;
		printf( "\n\n" ) ;

	}while( !( p[0] == 'n' || p[0] == 'N' )) ;
}


/*------------------------------------------------------------------------------
	initialize : original version
------------------------------------------------------------------------------*/
void init_func_var_000( rPARAM_t *prtPARAM )
{
	randomize_time() ;

	prtPARAM->t_year = 10 ;
	prtPARAM->year = 1 ;

	prtPARAM->acre = 1000 ;
	prtPARAM->people = 100 ;
	prtPARAM->starved = 0 ;
	prtPARAM->come = 5 ;
	prtPARAM->harvest = 3 ;
	prtPARAM->eat = 200 ;
	prtPARAM->store = 2800 ;
	prtPARAM->p1 = 0.0 ;
	prtPARAM->d1 = 0 ;

	prtPARAM->plague = 1 ;
	prtPARAM->rest = 0 ;

	prtPARAM->rate = 17 + get_rand( 10 ) ;
}


/*------------------------------------------------------------------------------
	initialize : the other version
------------------------------------------------------------------------------*/
void init_func_var_001( rPARAM_t *prtPARAM )
{
	randomize_time() ;

	prtPARAM->t_year = 10 ;
	prtPARAM->year = 1 ;

	prtPARAM->acre = 1000 ;
	prtPARAM->people = 100 ;
	prtPARAM->starved = 0 ;
	prtPARAM->come = 5 ;
	prtPARAM->harvest = 3 ;
	prtPARAM->eat = 200 ;
	prtPARAM->store = 2800 ;
	prtPARAM->p1 = 0.0 ;
	prtPARAM->d1 = 0 ;

	prtPARAM->plague = 1 ;
	prtPARAM->rest = 0 ;

	prtPARAM->rate = 17 + get_rand( 6 ) ;
}


/*------------------------------------------------------------------------------
	initialize
------------------------------------------------------------------------------*/
void init( rPARAM_t *prtPARAM, char variation )
{
	(INITIALIZE[ variation ])( prtPARAM ) ;
}


/*------------------------------------------------------------------------------
	report
------------------------------------------------------------------------------*/
void report( rPARAM_t *prtPARAM, char flag )
{
	if ( flag )
	{
		printf( "\n\n%s:  %s",
			MES_STRING[_MES_NAME_], MES_STRING[_MES_REPORT_] ) ;
		printf( MES_STRING[_MES_YEAR_], prtPARAM->year ) ;
		printf( MES_STRING[_MES_STARVED_], prtPARAM->starved ) ;
		printf( MES_STRING[_MES_CAME_TO_], prtPARAM->come ) ;
		if ( prtPARAM->plague <= 0 )
		{
			printf( MES_STRING[_MES_PLAGUE_] ) ;
		}
		printf( MES_STRING[_MES_POPULATION_], prtPARAM->people ) ;
		printf( MES_STRING[_MES_NOW_OWNS_ACRES_], prtPARAM->acre ) ;
		printf( MES_STRING[_MES_HARVESTED_], prtPARAM->harvest ) ;
		printf( MES_STRING[_MES_RATS_], prtPARAM->eat ) ;
		printf( MES_STRING[_MES_STORE_], prtPARAM->store ) ;
	}else{
		printf( "\n" ) ;
		printf( MES_STRING[_MES_YEAR_], prtPARAM->year ) ;
		printf( MES_STRING[_MES_POPULATION_], prtPARAM->people ) ;
		printf( MES_STRING[_MES_NOW_OWNS_ACRES_], prtPARAM->acre_rest ) ;
		printf( MES_STRING[_MES_STORE_], prtPARAM->rest ) ;
	}
	printf( "\n" ) ;

}


/*------------------------------------------------------------------------------
	advice
------------------------------------------------------------------------------*/
void advice( rPARAM_t *prtPARAM )
{
	long	feed ;
	long	laborer ;
	long	rest ;
	long	trade ;
	long	plant ;
	long	land_after ;
	long	plant_after ;
	long	t0 ;

	if ( prtPARAM->advice_mode == 0x00 )
	{
		return ;	/* advice mode = off */
	}

	/* calculation of necessary bushels */
	/* for people */
	feed = prtPARAM->people * BUSHELS_PER_PERSON ;
	/* for the harvest  */
	laborer = prtPARAM->people * 10 ;
	plant = ( prtPARAM->acre > laborer ) ? laborer : prtPARAM->acre ;
	/* situation after trading */
	land_after = 0 ;
	plant_after = 0 ;
	rest = prtPARAM->store - feed ;

	t0 = rest - ( plant / 2 ) ;
	trade = (( t0 < 0 ) ? ( t0 - prtPARAM->rate ) : t0 ) / prtPARAM->rate ;

	if ( trade != 0 )
	{
		if ( trade < 0 )
		{
			trade-- ;
		}
		land_after = prtPARAM->acre + trade ;

		if ( land_after <= 0 )
		{
			trade = 1 - prtPARAM->acre ;
			land_after = 1 ;
		}

		rest = ( rest - trade * prtPARAM->rate ) * 2 ;
		plant_after = ( land_after < laborer ) ? land_after : laborer ;
		if ( rest < plant_after )
		{
			plant_after = rest ;
		}
	}

	/* display of numerical result  */
	printf( "\n* [ %s ]\n", MES_STRING[_MES_ADVICE_] ) ;

	if ( prtPARAM->advice_mode & 0x01 )
	{
		printf( "  " ) ;
		printf( MES_STRING[_MES_ADVICE_1_DEMAND_], feed ) ;
	}
	printf( "  " ) ;
	printf( MES_STRING[_MES_ADVICE_2_PLANT_], plant ) ;

	if ( prtPARAM->advice_mode & 0x03 )
	{
		if ( trade != 0 )
		{
			if ( trade < 0 )
			{
				printf( "  " ) ;
				printf( MES_STRING[_MES_ADVICE_3_TRADING_SELL_], -trade ) ;
			}else{
				if ( prtPARAM->advice_mode & 0x02 )
				{
					printf( "  " ) ;
					printf( MES_STRING[_MES_ADVICE_3_TRADING_BUY_], trade ) ;
				}
			}
			if ( trade < 0 || prtPARAM->advice_mode & 0x02 )
			{
				printf( "  " ) ;
				printf( MES_STRING[_MES_ADVICE_4_RESULT_LAND_], land_after ) ;
			}
		}

		if ( plant_after > 0 && plant_after != plant )
		{
			if ( trade < 0 || prtPARAM->advice_mode & 0x02 )
			{
				printf( "  " ) ;
				printf( MES_STRING[_MES_ADVICE_4_RESULT_PLANT_], plant_after ) ;
			}
		}
	}
	printf( "\n" ) ;
}


/*------------------------------------------------------------------------------
	numeric input
------------------------------------------------------------------------------*/
int input_num( rPARAM_t *prtPARAM, int mesno )
{
	int		end_flag ;

	do{
		end_flag = 1 ;
		printf( MES_STRING[mesno] ) ;
		fgets( g_buf, sizeof( g_buf ), stdin ) ;

		if ( g_buf[0] == '?' )
		{
			/* repeat a report.. */
			report( prtPARAM, 0 ) ;
			printf( MES_STRING[_MES_TRADING_], prtPARAM->rate ) ;
			continue ;
		}
		end_flag = 0 ;

	}while( end_flag ) ;

	return atoi( g_buf ) ;
}


/*------------------------------------------------------------------------------
	string input
------------------------------------------------------------------------------*/
char *input_str( int mesno )
{
	printf( MES_STRING[mesno] ) ;
	fgets( g_buf, sizeof( g_buf ), stdin ) ;
	return g_buf ;
}


/*------------------------------------------------------------------------------
	parameter set
------------------------------------------------------------------------------*/
int set_param( rPARAM_t *prtPARAM )
{
	char	*p ;
	int		end_flag ;

	printf( MES_STRING[_MES_TRADING_], prtPARAM->rate ) ;

	advice( prtPARAM ) ;

	do{
		prtPARAM->rest = prtPARAM->store ;
		prtPARAM->acre_rest = prtPARAM->acre ;

		/* buy */
		do{
			end_flag = 1 ;
			prtPARAM->trade = input_num( prtPARAM, _MES_BUY_ ) ;
			if ( prtPARAM->trade == 0 )
			{
				do{
					end_flag = 1 ;
					/* sell */
					prtPARAM->trade = input_num( prtPARAM, _MES_SELL_ ) ;
					if ( prtPARAM->trade >= prtPARAM->acre )
					{
						/* think again.. */
						printf( "%s:  ", MES_STRING[_MES_NAME_] ) ;
						printf( MES_STRING[_MES_THINK_AGAIN_] ) ;
						printf( MES_STRING[_MES_NO_ACRES_], prtPARAM->acre ) ;
						continue ;
					}else if ( prtPARAM->trade < 0 )
					{
						/* abort.. */
						printf( "\n%s", MES_STRING[_MES_ABORT_1_] ) ;
						printf( MES_STRING[_MES_ABORT_2_] ) ;
						return -1 ;
					}
					end_flag = 0 ;

				}while( end_flag ) ;

				prtPARAM->trade = -prtPARAM->trade ;

			}else if ( prtPARAM->trade > prtPARAM->rest / prtPARAM->rate )
			{
				/* think again.. */
				printf( "%s:  ", MES_STRING[_MES_NAME_] ) ;
				printf( MES_STRING[_MES_THINK_AGAIN_] ) ;
				printf( MES_STRING[_MES_NO_BUSHELS_], prtPARAM->store ) ;
				continue ;
			}else if ( prtPARAM->trade < 0 )
			{
				/* abort.. */
				printf( "\n%s", MES_STRING[_MES_ABORT_1_] ) ;
				printf( MES_STRING[_MES_ABORT_2_] ) ;
				return -1 ;
			}
			end_flag = 0 ;

		}while( end_flag ) ;

		printf( "\n" ) ;

		prtPARAM->acre_rest += prtPARAM->trade ;
		prtPARAM->rest -= ( prtPARAM->trade * prtPARAM->rate ) ;

		do{
			end_flag = 1 ;
			prtPARAM->feed = input_num( prtPARAM, _MES_FEED_ ) ;

			if ( prtPARAM->feed > prtPARAM->rest )
			{
				/* think again.. */
				printf( "%s:  ", MES_STRING[_MES_NAME_] ) ;
				printf( MES_STRING[_MES_THINK_AGAIN_] ) ;
				printf( MES_STRING[_MES_NO_BUSHELS_], prtPARAM->rest ) ;
				continue ;
			}else if ( prtPARAM->feed < 0 )
			{
				/* abort.. */
				printf( "\n%s", MES_STRING[_MES_ABORT_1_] ) ;
				printf( MES_STRING[_MES_ABORT_2_] ) ;
				return -1 ;
			}
			end_flag = 0 ;

		}while( end_flag ) ;

		printf( "\n" ) ;

		prtPARAM->rest -= prtPARAM->feed ;

		do{
			end_flag = 1 ;
			prtPARAM->plant = input_num( prtPARAM, _MES_PLANT_ ) ;

			if ( prtPARAM->plant > prtPARAM->acre_rest )
			{
				/* think again.. */
				printf( "%s:  ", MES_STRING[_MES_NAME_] ) ;
				printf( MES_STRING[_MES_THINK_AGAIN_] ) ;
				printf( MES_STRING[_MES_NO_ACRES_], prtPARAM->acre_rest ) ;
				continue ;
			}else if ( prtPARAM->plant > prtPARAM->rest * 2 )
			{
				/* think again.. */
				printf( "%s:  ", MES_STRING[_MES_NAME_] ) ;
				printf( MES_STRING[_MES_THINK_AGAIN_] ) ;
				printf( MES_STRING[_MES_NO_BUSHELS_], prtPARAM->rest ) ;
				continue ;
			}else if ( prtPARAM->plant > prtPARAM->people * 10 )
			{
				printf( MES_STRING[_MES_NO_PEOPLE_], prtPARAM->people ) ;
				continue ;
			}else if ( prtPARAM->plant < 0 )
			{
				/* abort.. */
				printf( "\n%s", MES_STRING[_MES_ABORT_1_] ) ;
				printf( MES_STRING[_MES_ABORT_2_] ) ;
				return -1 ;
			}
			end_flag = 0 ;

		}while( end_flag ) ;

		printf( "\n\n" ) ;
		p = input_str(_MES_SURE_) ;

	}while(( p[0] == 'n' || p[0] == 'N' )) ;

	return 1 ;
}


/*------------------------------------------------------------------------------
	execute : original version
------------------------------------------------------------------------------*/
int exec_func_var_000( rPARAM_t *prtPARAM )
{
	long t0 ;

	/* starved.. */
	t0 = prtPARAM->feed / BUSHELS_PER_PERSON ;
	prtPARAM->starved = ( t0 < prtPARAM->people ) ? prtPARAM->people - t0 : 0 ;

	/* When 45% or more of the people starves.. */
	if ( prtPARAM->starved * 20 > prtPARAM->people * 9 )
	{
		printf( "\n" ) ;
		printf( MES_STRING[_MES_GAMEOVER_1_], prtPARAM->starved ) ;
		printf( "%s%s%s",
			MES_STRING[_MES_GAMEOVER_2_],
			MES_STRING[_MES_GAMEOVER_3_],
			MES_STRING[_MES_GAMEOVER_4_] ) ;
		return -1 ;
	}

	/* average of starved people per year..
		P1=((Z-1)*P1+D*100/P)/Z
	*/
	prtPARAM->p1 = ((( prtPARAM->year - 1 ) * prtPARAM->p1 )
		+ ( prtPARAM->starved * 100 ) / prtPARAM->people ) / prtPARAM->year ;
	prtPARAM->d1 += prtPARAM->starved ;

	/* plant seed & harvest.. */
	prtPARAM->harvest = 1 + get_rand( 5 ) ;
	prtPARAM->rest -= ( prtPARAM->plant >> 1 ) ;

	/* rat ate..
		1/5 ... eat half.
		1/5 ... eat quarter.
		3/5 ... don't eat. 
	*/ 
	t0 = get_rand( 5 ) ;
	prtPARAM->eat = t0 > 1 ? 0 : prtPARAM->rest >> ( t0 + 1 ) ;
	prtPARAM->store =
		prtPARAM->rest - prtPARAM->eat
		+ ( prtPARAM->plant * prtPARAM->harvest ) ;

	/* plague occurs at the probability of 20%.. */
	prtPARAM->plague = get_rand( 20 ) - 3 ;

	/* people came to city.. */
	t0 = 1 + get_rand( 5 ) ;
	prtPARAM->come =
		(( t0 * 20 * prtPARAM->acre + prtPARAM->store )
		/ prtPARAM->people / 100 ) + 1 ;

	prtPARAM->people += ( prtPARAM->come - prtPARAM->starved ) ;

	if ( prtPARAM->plague <= 0 )
	{
		prtPARAM->people /= 2 ;
	}

	prtPARAM->acre = prtPARAM->acre_rest ;
	prtPARAM->rate = 17 + get_rand( 10 ) ;

	if ( prtPARAM->year++ >= prtPARAM->t_year )
	{
		prtPARAM->acre_per_people = (float)prtPARAM->acre / prtPARAM->people ;
		prtPARAM->discontent =
			( prtPARAM->people * 8 * ( get_rand( 10 ) + 1 )) / 100 ;
		return 0 ;
	}

	return 1 ;
}


/*------------------------------------------------------------------------------
	execute : the other version
------------------------------------------------------------------------------*/
int exec_func_var_001( rPARAM_t *prtPARAM )
{
	/* starved.. */
	prtPARAM->starved =
		prtPARAM->people - ( prtPARAM->feed / BUSHELS_PER_PERSON ) ;
	if ( prtPARAM->starved >= 0 )
	{
		prtPARAM->come = 0 ;
	}else{
		prtPARAM->come = -prtPARAM->starved / 2 ;
		prtPARAM->starved = 0 ;
	}

	/* When 45% or more of the people starves.. */
	if ( prtPARAM->starved * 20 > prtPARAM->people * 9 )
	{
		printf( "\n" ) ;
		printf( MES_STRING[_MES_GAMEOVER_1_], prtPARAM->starved ) ;
		printf( "%s%s%s",
			MES_STRING[_MES_GAMEOVER_2_],
			MES_STRING[_MES_GAMEOVER_3_],
			MES_STRING[_MES_GAMEOVER_4_] ) ;
		return -1 ;
	}

	/* average of starved people per year..
		P1=((Z-1)*P1+D*100/P)/Z
	*/
	prtPARAM->p1 = ((( prtPARAM->year - 1 ) * prtPARAM->p1 )
		+ ( prtPARAM->starved * 100 ) / prtPARAM->people ) / prtPARAM->year ;

	prtPARAM->people -= prtPARAM->starved ;
	prtPARAM->d1 += prtPARAM->starved ;

	/* plant seed & harvest.. */
	prtPARAM->harvest = 1 + get_rand( 5 ) ;
	prtPARAM->rest -= ( prtPARAM->plant >> 1 ) ;

	/* rat ate..
		( store + ( plant * harvest )) * 0.07 * RND(1)
	*/ 
	prtPARAM->rest += ( prtPARAM->plant * prtPARAM->harvest ) ;
	prtPARAM->eat = ( prtPARAM->rest * get_rand( 700 )) / 10000 ;
	prtPARAM->store = prtPARAM->rest - prtPARAM->eat ;

	/* plague occurs at the probability of 10%.. */
	prtPARAM->plague = get_rand( 11 ) ;

	/* people came to city.. */
	prtPARAM->come += ( 5 - prtPARAM->harvest ) * prtPARAM->store / 600 + 1 ;
	if ( prtPARAM->come > 50 )
	{
		prtPARAM->come = 50 ;
	}
	if ( prtPARAM->come < 0 )
	{
		prtPARAM->come = 0 ;
	}
	prtPARAM->people += prtPARAM->come ;

	if ( prtPARAM->plague <= 0 )
	{
		prtPARAM->people /= 2 ;
	}

	prtPARAM->acre = prtPARAM->acre_rest ;
	prtPARAM->rate = 17 + get_rand( 6 ) ;

	if ( prtPARAM->year++ >= prtPARAM->t_year )
	{
		prtPARAM->acre_per_people = (float)prtPARAM->acre / prtPARAM->people ;
		prtPARAM->discontent =
			( prtPARAM->people * 8 * ( get_rand( 10 ) + 1 )) / 100 ;
		return 0 ;
	}

	return 1 ;
}


/*------------------------------------------------------------------------------
	execute
------------------------------------------------------------------------------*/
int execute( rPARAM_t *prtPARAM, char variation )
{
	return (EXECUTE[ variation ])( prtPARAM ) ;
}


/*------------------------------------------------------------------------------
	ending
------------------------------------------------------------------------------*/
void ending( rPARAM_t *prtPARAM )
{
	printf( MES_STRING[_MES_END_1_1_], prtPARAM->t_year ) ;
	printf( MES_STRING[_MES_END_1_2_], prtPARAM->p1 ) ;
	printf( MES_STRING[_MES_END_1_3_] ) ;
	printf( MES_STRING[_MES_END_1_4_], prtPARAM->d1 ) ;

	printf( MES_STRING[_MES_END_2_1_], prtPARAM->t_app ) ;
	printf( MES_STRING[_MES_END_2_2_], prtPARAM->acre_per_people ) ;

	if ( prtPARAM->t_year >= 10 )
	{
		printf( "\n" ) ;

		if ( prtPARAM->p1 > 33 || prtPARAM->acre_per_people < 7 )
		{
			printf( "%s%s%s",
				MES_STRING[_MES_GAMEOVER_2_],
				MES_STRING[_MES_GAMEOVER_3_],
				MES_STRING[_MES_GAMEOVER_4_] ) ;
		}else
		if ( prtPARAM->p1 > 10 || prtPARAM->acre_per_people < 9 )
		{
			printf( "%s%s%s",
				MES_STRING[_MES_ENDING_2_1_],
				MES_STRING[_MES_ENDING_2_2_],
				MES_STRING[_MES_ENDING_2_3_] ) ;
		}else
		if ( prtPARAM->p1 > 3 || prtPARAM->acre_per_people < 10 )
		{
			printf( MES_STRING[_MES_ENDING_3_1_] ) ;
			printf( MES_STRING[_MES_ENDING_3_2_], prtPARAM->discontent ) ;
			printf( "%s%s",
				MES_STRING[_MES_ENDING_3_3_],
				MES_STRING[_MES_ENDING_3_4_] ) ;
		}else{
			printf( "%s%s",
				MES_STRING[_MES_ENDING_1_1_],
				MES_STRING[_MES_ENDING_1_2_] ) ;
		}
	}
}
